"""Name-US: CV-Blend Takes
Description-US: Creates an arbitrary number of takes blending the values between two selected takes.

LICENSE
-------

Copyright (C) 2015 Maxon Computer, Inc.
Programming: Rick Barrett <rick@maxon.net>
Documentation: Donovan Keith <donovanskeith@gmail.com>

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


USAGE INSTRUCTIONS
------------------

1. Select (exactly) two takes with similar overrides.
2. Run the Script.
3. Enter the number of blends between takes (default is 3)
4. Press OK

"""

import c4d

def getDescIdName(op,descID):
    description = op.GetDescription(c4d.DESCFLAGS_DESC_0)
    for bc, paramid, groupid in description:
        if paramid == descID:
            return bc[c4d.DESC_NAME]

def main():
    # Get the TakeData
    td = doc.GetTakeData()

    # Get the Selected Takes
    sel = td.GetTakeSelection(False)
    # Error if there's not exactly 2 takes selected
    if len(sel) != 2:
        c4d.gui.MessageDialog("Please select exactly two takes")
        return None

    # Ask the user how many blend iterations they want
    blendCount = int(c4d.gui.InputDialog("Blend Count", "3"))

    # Init the list that will contain blend overrides
    blendList = []

    # Get a list of overrides in the first selected take
    firstOver = sel[0].GetOverrides()

    # Loop through overrides
    for over in firstOver:
        # Loop through DescIDs
        for firstDescID in over.GetAllOverrideDescID():
            # Get this value and make sure it's blendable (int or float)
            firstVal = over.GetParameter(firstDescID,c4d.DESCFLAGS_GET_0)
            if isinstance(firstVal, (int,float)):
                # Find this DescID in the second selected take
                secondOver, secondTake = sel[1].FindOverrideInHierarchy(td, over.GetSceneNode(), firstDescID)
                # If the override exists, append to the blend list
                # [0] The Node
                # [1] The DescID
                # [2] The Value in the First Take
                # [3] The Value in the Second Take
                if secondOver:
                    blendList.append([over.GetSceneNode(),
                                      firstDescID,
                                      firstVal,
                                      secondOver.GetParameter(firstDescID, c4d.DESCFLAGS_GET_0)])

    # Iterate over the desired blend range
    name = ""
    for takeIdx in xrange(0,blendCount):
        # Create a take
        blendTake = td.AddTake("%s_%s" % (sel[0].GetName(), takeIdx + 1), None, None)
        td.InsertTake(blendTake, sel[1], c4d.INSERT_BEFORE)

        # Iterate over blend list
        for over in blendList:
            # Determine the blend value
            val = (((over[3] - over[2]) / (blendCount + 1)) * (takeIdx + 1)) + over[2]

            # Create the override
            override = blendTake.FindOrAddOverrideParam(td, over[0], over[1], val)
            name += "%s-%s " % (getDescIdName(over[0],over[1]),val)
        # Set the Take Name based on Overrides
        blendTake.SetName(name)
        name = ""

    # Force the UI to update
    c4d.EventAdd()


if __name__ == '__main__':
    main()
